//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013-2017 Nicolas Froment, Joachim Schmitz
//  Copyright (C) 2014 Jörn Eichler
//  Edited and Changed by: Ziya Mete Demircan
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.2
import MuseScore 3.0

MuseScore {
      version:  "3.0"
      description: qsTr("This plugin colors notes in the Circle of Fifths")
      menuPath: "Plugins.Notes.Color Notes-sharp-flat"
      
              property string noteCn : "#000000"; // c
              property string noteGn : "#000000"; // g
              property string noteDn : "#000000"; // d
              property string noteAn : "#000000"; // a
              property string noteEn : "#000000"; // e
              property string noteBn : "#000000"; // b
              property string noteFn : "#000000"; // f
              property string noteCb : "#0000ff"; // cb
              property string noteCs : "#ff0000"; // c#
              property string noteDb : "#0000ff"; // db
              property string noteDs : "#ff0000"; // d#
              property string noteEb : "#0000ff"; // eb
              property string noteEs : "#ff0000"; // e#
              property string noteFb : "#0000ff"; // fb 
              property string noteFs : "#ff0000"; // f# 
              property string noteGb : "#0000ff"; // gb 
              property string noteGs : "#ff0000"; // g# 
              property string noteAb : "#0000ff"; // ab
              property string noteAs : "#ff0000"; // a# 
              property string noteBb : "#0000ff"; // bb
              property string noteBs : "#ff0000"; // b#/c 
			  
              property string black  : "#000000"; // black
              property string red  : "#ff0000"; // red
              property string blue  : "#0000ff"; // blue


property variant colors : [  noteFn, noteCn, noteGn, noteDn, noteAn, noteEn, noteBn, noteFs, noteCs, noteGs, noteDs, noteAs, noteEs, noteBs, noteGn, noteDn, noteAn, noteEn, noteBn, noteFs, noteCs, noteAs, noteEb, noteBb, noteFn, noteCn, noteGn, noteDn, noteAn, noteEn, noteBn, noteFs, noteCn, noteDs, noteBb ]

      // Apply the given function to all notes in selection
      // or, if nothing is selected, in the entire score
	  //  https://musescore.org/en/handbook/developers-handbook/plugin-development/tonal-pitch-class-enum

      function applyToNotesInSelection(func) {
            var cursor = curScore.newCursor();
            cursor.rewind(1);
            var startStaff;
            var endStaff;
            var endTick;
            var fullScore = false;
            if (!cursor.segment) { // no selection
                  fullScore = true;
                  startStaff = 0; // start with 1st staff
                  endStaff = curScore.nstaves - 1; // and end with last
            } else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2);
                  if (cursor.tick === 0) {
                        // this happens when the selection includes
                        // the last measure of the score.
                        // rewind(2) goes behind the last segment (where
                        // there's none) and sets tick=0
                        endTick = curScore.lastSegment.tick + 1;
                  } else {
                        endTick = cursor.tick;
                  }
                  endStaff = cursor.staffIdx;
            }
            //console.log(startStaff + " - " + endStaff + " - " + endTick)
            for (var staff = startStaff; staff <= endStaff; staff++) {
                  for (var voice = 0; voice < 4; voice++) {
                        cursor.rewind(1); // sets voice to 0
                        cursor.voice = voice; //voice has to be set after goTo
                        cursor.staffIdx = staff;

                        if (fullScore)
                              cursor.rewind(0) // if no selection, beginning of score

                        while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                              if (cursor.element && cursor.element.type === Element.CHORD) {
                                    var graceChords = cursor.element.graceNotes;
                                    for (var i = 0; i < graceChords.length; i++) {
                                          // iterate through all grace chords
                                          var graceNotes = graceChords[i].notes;
                                          for (var j = 0; j < graceNotes.length; j++)
                                                func(graceNotes[j]);
                                    }
                                    var notes = cursor.element.notes;
                                    for (var k = 0; k < notes.length; k++) {
                                          var note = notes[k];
                                          func(note);
                                    }
                              }
                              cursor.next();
                        }
                  }
            }
      }

      function colorNote(note) {
            
			if (note.color == black) {
                  note.color = colors[note.tpc+1];
				  console.log("103, Note Color: note.tpc: "+ note.tpc + " pitch:" + note.pitch);
            } else {
                  note.color = black;
            } 
            
			if (note.accidental) {
				if (note.accidental.color == black) {
					note.accidental.color = colors[note.tpc+1];
					console.log("111, Note Accidental: note:"+ note + " pitch:" + note.pitch + " accidental:" + note.accidental);
					} else {
					note.accidental.color = blue;
					}
			}
			
			
            for (var i = 0; i < note.dots.length; i++) {
                  if (note.dots[i]) {
                        if (note.dots[i].color == black) {
                              note.dots[i].color = colors[note.tpc+1];
					console.log("122, Note Dots: note:"+ note.dots[i] + " color:" + colors[note.tpc+1]);
                        } else {
                              note.dots[i].color = black;
                        }
                  }
			}
         }

      onRun: {
            console.log("hello colornotes");

            if (typeof curScore === 'undefined')
                  Qt.quit();

            applyToNotesInSelection(colorNote)

            Qt.quit();
         }
}
